﻿ALTER TABLE mbta.mbta_lines ADD COLUMN length float8; 
UPDATE mbta.mbta_lines SET length = ST_Length(geom);

-- Airport to Harvard
SELECT N.station, E.route, E.line, D.* 
FROM 
    pgr_dijkstra(
        'SELECT id, source, target, length As cost FROM mbta.mbta_lines', 
        (SELECT id FROM mbta.mbta_lines_vertices_pgr WHERE station = 'Airport'),
        (SELECT id FROM mbta.mbta_lines_vertices_pgr WHERE station = 'Harvard'),
        directed := FALSE
    ) AS D INNER JOIN 
    mbta.mbta_lines_vertices_pgr N ON N.id = D.node LEFT JOIN 
    mbta.mbta_lines E ON D.edge = E.id
;

-- Airport to Mattapan
SELECT N.station, E.route, E.line, D.* 
FROM 
    pgr_dijkstra(
        'SELECT id, source, target, length As cost FROM mbta.mbta_lines', 
        (SELECT id FROM mbta.mbta_lines_vertices_pgr WHERE station = 'Airport'),
        (SELECT id FROM mbta.mbta_lines_vertices_pgr WHERE station = 'Mattapan'),
        directed := FALSE
    ) AS D INNER JOIN 
    mbta.mbta_lines_vertices_pgr N ON N.id = D.node LEFT JOIN 
    mbta.mbta_lines E ON D.edge = E.id
;

-- Find node IDs for State Street and Government Center: In our case Government Center = 1 and State Street = 2
SELECT id, station FROM mbta.mbta_lines_vertices_pgr;
SELECT id, station FROM mbta.mbta_lines_vertices_pgr WHERE station ILIKE '%government%' OR station ILIKE '%state%';

SELECT N.station, E.route, E.line, D.* 
FROM 
    pgr_dijkstra(
        'SELECT id, source, target, length AS cost FROM mbta.mbta_lines',
        1,
        2, 
        directed := FALSE
    ) AS D INNER JOIN 
    mbta.mbta_lines_vertices_pgr N ON N.id = D.node LEFT JOIN 
    mbta.mbta_lines AS E ON D.edge = E.id
;

-- Suppose disabled train stuck between Government Center and State St

SELECT N.station, E.route, E.line, D.* 
FROM pgr_dijkstra(
    'SELECT id, source, target, length AS cost FROM mbta.mbta_lines WHERE id <> 3', 
    1,
    2,
    directed := FALSE
) AS D INNER JOIN mbta.mbta_lines_vertices_pgr N ON N.id = D.node LEFT JOIN mbta.mbta_lines AS E ON D.edge = E.id;

DROP TABLE IF EXISTS mbta.mbta_restrictions;

CREATE TABLE mbta.mbta_restrictions (
    rid serial primary key, 
    to_edge integer, 
    via_path text, 
    to_cost double precision
);

INSERT INTO mbta.mbta_restrictions (to_edge, via_path, to_cost)
WITH X AS (SELECT id FROM mbta.mbta_lines_vertices_pgr WHERE station IN ('Government Center','Park Street'))
SELECT id, NULL, 10000000 --Don't use negative cost
FROM mbta.mbta_lines 
WHERE source IN (SELECT id FROM X) AND target IN(SELECT id FROM X);

SELECT * FROM mbta.mbta_restrictions;

-- From Airport to Harvard No Restriction
SELECT T.seq, E.id As edge, N.id As node, N.station, E.line, E.route, T.cost, SUM(T.cost) OVER(ORDER BY T.seq) As cost_total
FROM 
    pgr_trsp(
        'SELECT 
            id::int4,
            source::int4,
            target::int4,
            length::float8 AS cost
        FROM mbta.mbta_lines',
        (SELECT id FROM mbta.mbta_lines_vertices_pgr WHERE station = 'Airport')::int,
        (SELECT id FROM mbta.mbta_lines_vertices_pgr WHERE station = 'Harvard')::int,
        directed := FALSE, 
        has_rcost := FALSE) T INNER JOIN 
    mbta.mbta_lines_vertices_pgr N ON T.id1 = N.id LEFT JOIN 
    mbta.mbta_lines E ON T.id2 = E.id
ORDER BY T.seq;

-- From Airport to Harvard with Restrictions
SELECT T.seq, E.id As edge, N.id As node, N.station, E.line, E.route, T.cost, SUM(T.cost) OVER(ORDER BY T.seq) As cost_total
FROM 
    pgr_trsp(
        'SELECT 
            id::int4,
            source::int4,
            target::int4,
            length::float8 AS cost
        FROM mbta.mbta_lines',
        (SELECT id FROM mbta.mbta_lines_vertices_pgr WHERE station = 'Airport')::int,
        (SELECT id FROM mbta.mbta_lines_vertices_pgr WHERE station = 'Harvard')::int,
        FALSE, 
        FALSE,
        'SELECT to_edge AS target_id, via_path, to_cost FROM mbta.mbta_restrictions'
    ) T INNER JOIN 
    mbta.mbta_lines_vertices_pgr N ON T.id1 = N.id LEFT JOIN 
    mbta.mbta_lines E ON T.id2 = E.id
ORDER BY T.seq;

-- KSP

SELECT D.path_id, D.path_seq, D.edge, N.station, E.line, E.geom 
FROM 
    pgr_ksp(
        'SELECT id, source, target, length AS cost FROM mbta.mbta_lines',
        1,
        2,
        10, 
        directed := FALSE) AS D INNER JOIN 
        mbta.mbta_lines_vertices_pgr N ON N.id = D.node LEFT JOIN 
        mbta.mbta_lines E ON D.edge = E.id
;
