﻿-- run pois.sql first -- we'll be using 2.5 dev VRP - pick and Deliver function -- http://docs.pgrouting.org/dev/en/pgr_pickDeliver.html
-- We'll record time using epoch/1 so we get a time interval that starts from Jan 1970 and each unit is 1 minute (instead of 1 second) people expect to be picked up within 60 minutes of their arrival, takes 5 minutes to board on our taxi service

DROP TABLE IF EXISTS cost_matrix;

SELECT * 
INTO cost_matrix 
FROM 
    pgr_dijkstraCostMatrix(
        'SELECT gid AS id, source, target, cost_s*3 AS cost, reverse_cost_s*3 AS reverse_cost FROM osm.ways',
        (SELECT array_agg(DISTINCT closest_node_id) FROM pois)
);


DROP TABLE IF EXISTS demands;

CREATE TABLE demands (
    id serial primary key, 
    p_node_id integer, 
    demand integer, 
    p_open float, 
    p_close float,  
    p_service float,  
    d_node_id integer, 
    d_open float, 
    d_close float, 
    d_service float
);

INSERT INTO demands (p_node_id, demand, p_open, p_close, p_service, d_node_id, d_open, d_close, d_service)
SELECT 
    p.closest_node_id AS p_node_id, 
    2, 
    extract(epoch FROM tm) AS p_open, 
    extract(epoch FROM (tm + interval '10 min')) AS p_close, 
    0 AS p_service, 
    d.closest_node_id AS d_node_id, 
    extract(epoch FROM tm) AS d_open, 
    extract(epoch FROM (tm + interval '90 min')) AS d_close, 
    0 AS d_service
FROM 
    (SELECT closest_node_id FROM pois WHERE name = 'Airport') As p, 
    (SELECT closest_node_id FROM pois WHERE name = 'Seaport Hotel & World Trade Center') AS d,
    unnest('{2017-8-16 8:00 AM}'::timestamptz[]) AS tm;

INSERT INTO demands (p_node_id, demand, p_open, p_close, p_service, d_node_id, d_open, d_close, d_service)
SELECT 
    p.closest_node_id AS p_node_id, 
    1, 
    extract(epoch FROM tm) AS p_open, 
    extract(epoch FROM (tm + interval '10 min')) AS p_close, 
    0 AS p_service, 
    d.closest_node_id AS d_node_id, 
    extract(epoch FROM tm) AS d_open, 
    extract(epoch FROM (tm + interval '90 min')) AS d_close, 
    0 AS d_service
FROM 
    (SELECT closest_node_id FROM pois WHERE name = 'Airport') As p, 
    (SELECT closest_node_id FROM pois WHERE name = 'Seaport Hotel & World Trade Center') AS d,
    unnest('{2017-8-16 8:00 AM}'::timestamptz[]) AS tm;

INSERT INTO demands (p_node_id, demand, p_open, p_close, p_service, d_node_id, d_open, d_close, d_service)
SELECT 
    p.closest_node_id AS p_node_id, 
    1, 
    extract(epoch FROM tm) AS p_open, 
    extract(epoch FROM (tm + interval '10 min')) AS p_close, 
    0 AS p_service, 
    d.closest_node_id AS d_node_id, 
    extract('epoch' FROM tm) AS d_open, 
    extract(epoch FROM (tm + interval '90 min')) AS d_close, 
    0 AS d_service
FROM 
    (SELECT closest_node_id FROM pois WHERE name = 'Airport') As p, 
    (SELECT closest_node_id FROM pois WHERE name = 'Seaport Hotel & World Trade Center') AS d,
    unnest('{2017-8-16 8:05 AM}'::timestamptz[]) AS tm

UNION ALL

SELECT 
    p.closest_node_id AS p_node_id, 
    1, 
    extract('epoch' FROM tm)/1 AS p_open, 
    extract(epoch FROM (tm + interval '60 minutes'))/1 AS p_close, 
    5 AS p_service,
    d.closest_node_id AS d_node_id,  
    extract('epoch' FROM tm)/1 AS d_open, 
    extract(epoch FROM (tm + interval '100 minutes'))/1 AS d_close, 
    5 AS d_service

FROM 
    (SELECT closest_node_id FROM pois WHERE name = 'Seaport Hotel & World Trade Center') AS p, 
    (SELECT closest_node_id FROM pois WHERE name = 'MIT') As d, 
    unnest('{2017-8-16 6:10 PM, 2017-8-16 8:00 PM, 2017-8-16 8:30 PM}'::timestamptz[]) AS tm
UNION ALL

SELECT 
    p.closest_node_id AS p_node_id, 
    1, 
    extract('epoch' FROM tm)/1 AS p_open, 
    extract(epoch FROM (tm + interval '60 minutes'))/1 AS p_close, 
    5 AS p_service,
    d.closest_node_id AS d_node_id,  
    extract('epoch' FROM tm)/1 AS d_open, 
    extract(epoch FROM (tm + interval '100 minutes'))/1 AS d_close, 
    5 AS d_service

FROM 
    (SELECT closest_node_id FROM pois WHERE name = 'Harpoon Brewery') AS p, 
    (SELECT closest_node_id FROM pois WHERE name = 'MIT') As d, 
    unnest('{2017-8-16 6:00 PM, 2017-8-16 6:05 PM, 2017-8-16 7:15 PM}'::timestamptz[]) AS tm
UNION ALL

SELECT 
    p.closest_node_id AS p_node_id, 
    1, 
    extract('epoch' FROM tm)/1 AS p_open, 
    extract(epoch FROM (tm + interval '60 minutes'))/1 AS p_close, 
    5 AS p_service,
    d.closest_node_id AS d_node_id,  
    extract('epoch' FROM tm)/1 AS d_open, 
    extract(epoch FROM (tm + interval '100 minutes'))/1 AS d_close, 
    5 AS d_service

FROM 
    (SELECT closest_node_id FROM pois WHERE name = 'Harpoon Brewery') AS p, 
    (SELECT closest_node_id FROM pois WHERE name = 'Harvard CGIS Knafel') As d, 
    unnest('{2017-8-16 5:15 PM,2017-8-16 7:15 PM}'::timestamptz[]) AS tm
UNION ALL

SELECT 
    p.closest_node_id AS p_node_id, 
    1, 
    extract('epoch' FROM tm)/1 AS p_open, 
    extract(epoch FROM (tm + interval '30 minutes'))/1 AS p_close, 
    5 AS p_service,
    d.closest_node_id AS d_node_id,  
    extract('epoch' FROM tm)/1 AS d_open, 
    extract(epoch FROM (tm + interval '120 minutes'))/1 AS d_close, 
    5 AS d_service
FROM 
    (SELECT closest_node_id FROM pois WHERE name = 'MIT') AS p, 
    (SELECT closest_node_id FROM pois WHERE name = 'Harvard CGIS Knafel') As d, 
    unnest('{2017-8-16 7:00 AM,2017-8-16 11:00 AM}'::timestamptz[]) AS tm

UNION ALL
SELECT 
    p.closest_node_id AS p_node_id, 
    1, 
    extract('epoch' FROM tm)/1 AS p_open, 
    extract(epoch FROM (tm + interval '60 minutes'))/1 AS p_close, 
    5 AS p_service,
    d.closest_node_id AS d_node_id,
    extract('epoch' FROM tm)/1 AS d_open, 
    extract(epoch FROM (tm + interval '100 minutes'))/1 AS d_close, 
    5 AS d_service
FROM 
    (SELECT closest_node_id FROM pois WHERE name = 'Seaport Hotel & World Trade Center') AS p, 
    (SELECT closest_node_id FROM pois WHERE name = 'Airport') As d, 
    unnest('{2017-8-16 10:00 PM, 2017-8-16 11:00 PM, 2017-8-16 9:10 PM, 2017-8-16 6:00 PM, 2017-8-16 6:00 PM}'::timestamptz[]) AS tm

UNION ALL
SELECT 
    p.closest_node_id AS p_node_id, 
    3, -- Attendees go to Brewery in Threesomes
    extract('epoch' FROM tm)/1 AS p_open, 
    extract(epoch FROM (tm + interval '60 minutes'))/1 AS p_close, 
    5 AS p_service,
    d.closest_node_id AS d_node_id,  
    extract('epoch' FROM tm)/1 AS d_open, 
    extract(epoch FROM (tm + interval '100 minutes'))/1 AS d_close, 
    5 AS d_service
FROM 
    (SELECT closest_node_id FROM pois WHERE name = 'Seaport Hotel & World Trade Center') AS p, 
    (SELECT closest_node_id FROM pois WHERE name = 'Harpoon Brewery') As d, 
    unnest('{2017-8-16 6:00 PM, 2017-8-16 7:00 PM, 2017-8-16 8:15 PM}'::timestamptz[]) AS tm

UNION ALL

SELECT 
    p.closest_node_id AS p_node_id, 
    2, -- Attendees  leave Brewery for Hotel in Twosomes
    extract('epoch' FROM tm)/1 AS p_open, 
    extract(epoch FROM (tm + interval '60 minutes'))/1 AS p_close, 
    5 AS p_service,
    d.closest_node_id AS d_node_id,  
    extract('epoch' FROM tm)/1 AS d_open, 
    extract(epoch FROM (tm + interval '100 minutes'))/1 AS d_close, 
    5 AS d_service

FROM 
    (SELECT closest_node_id FROM pois WHERE name = 'Harpoon Brewery') AS p, 
    (SELECT closest_node_id FROM pois WHERE name = 'Seaport Hotel & World Trade Center') As d, 
    unnest('{2017-8-16 9:15 PM}'::timestamptz[]) AS tm
;

-- One luxury limos, ten Spring vans available for the opening day of FOSS4G (Not true)

DROP TABLE IF EXISTS vehicles;

CREATE TABLE vehicles (
    id serial primary key, 
    capacity integer, 
    speed integer, 
    start_node_id integer, 
    start_open float, 
    start_close float, 
    start_service float, 
    end_node_id integer, 
    end_open float, 
    end_close float, 
    end_service float
);

TRUNCATE TABLE vehicles;

INSERT INTO vehicles (capacity, speed, start_node_id, start_open, start_close, start_service, end_node_id, end_open, end_close, end_service)
SELECT 
    4, 
    1 As speed, 
    se.closest_node_id AS start_node_id,
    extract('epoch' FROM '2017-8-16 6:00 AM'::timestamptz)/1 As start_open, 
    extract('epoch' FROM '2017-8-16 11:45 PM'::timestamptz)/1 AS start_close, 
    5 AS start_service,
    se.closest_node_id AS end_node_id, 
    extract('epoch' FROM '2017-8-16 6:00 AM'::timestamptz)/1 As end_open, 
    extract('epoch' FROM '2017-8-16 11:45 PM'::timestamptz)/1 AS end_close, 
    5 AS end_service
FROM 
    (SELECT closest_node_id FROM pois WHERE name = 'Seaport Hotel & World Trade Center') AS se, 
    generate_series(1,2) AS cars
;




SELECT 
    vrp.seq, 
    vrp.vehicle_seq, 
    vrp.vehicle_id, 
    vrp.stop_seq, 
    vrp.stop_id, 
    p.name As stop_name, 
    vrp.order_id AS demand_id, 
    COALESCE(CASE vrp.stop_id WHEN demands.p_node_id THEN demands.demand WHEN demands.d_node_id THEN -demands.demand ELSE 0 END) AS pax_change,
    vrp.cargo As pax_on_board, 
    to_char(to_timestamp(arrival_time), 
    'HH12:MI AM') AS arrival_time,
    (wait_time/60.0)::numeric(12,2) AS wait_duration_minutes, 
    to_char( to_timestamp(departure_time), 
    'HH12:MI AM') AS departure_time, 
    (travel_time/60.0)::numeric(12,2) AS travel_duration_minutes
FROM 
    _pgr_pickDeliver(
        'SELECT * FROM demands',
        'SELECT * FROM vehicles', 
        'SELECT * FROM cost_matrix',
        factor := 1, 
        max_cycles := 20, 
        initial_sol := 4
    ) AS vrp 
    LEFT JOIN 
    pois AS p 
    ON vrp.stop_id = p.closest_node_id 
    LEFT JOIN demands 
    ON vrp.order_id = demands.id
ORDER BY vrp.seq;